/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

#ifdef USE_JPG
#include <SDL.h>
#include <jpeglib.h>

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
#define RMASK 0x000000ff
#define GMASK 0x0000ff00
#define BMASK 0x00ff0000
#else
#define RMASK 0x00ff0000
#define GMASK 0x0000ff00
#define BMASK 0x000000ff
#endif

typedef struct {
	struct jpeg_destination_mgr pub;
	SDL_RWops* ops;
	JOCTET* buffer;
} mymanager;

#define OUTPUT_CHUNK_SIZE 0x800

static void SDL_RW_init_destination(j_compress_ptr cinfo)
{
	mymanager* dest = (mymanager*) cinfo->dest;
	dest->buffer = (JOCTET*)
		(*cinfo->mem->alloc_small)((j_common_ptr) cinfo, JPOOL_IMAGE,
					   OUTPUT_CHUNK_SIZE * sizeof(JOCTET));
	dest->pub.next_output_byte = dest->buffer;
	dest->pub.free_in_buffer = OUTPUT_CHUNK_SIZE;
}

static boolean SDL_RW_empty_output_buffer(j_compress_ptr cinfo)
{
	mymanager* dest = (mymanager*) cinfo->dest;
	SDL_RWwrite(dest->ops, dest->buffer, 1, OUTPUT_CHUNK_SIZE);
	dest->pub.next_output_byte = dest->buffer;
	dest->pub.free_in_buffer = OUTPUT_CHUNK_SIZE;
	return TRUE;
}

static void SDL_RW_term_destination(j_compress_ptr cinfo)
{
	mymanager* dest = (mymanager*) cinfo->dest;
	size_t remain = OUTPUT_CHUNK_SIZE - dest->pub.free_in_buffer;
	if(remain > 0){
		SDL_RWwrite(dest->ops, dest->buffer, 1, remain);
	}
}

static void saveJPG_core(Uint8* base, int pitch, SDL_RWops* dst, 
			 int _w, int _h, int _q){
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	JSAMPROW row_pointer[1];
	static mymanager jdest;
  
	int h;
  
	jdest.pub.init_destination = SDL_RW_init_destination;
	jdest.pub.empty_output_buffer = SDL_RW_empty_output_buffer;
	jdest.pub.term_destination = SDL_RW_term_destination;
	jdest.ops = dst;
  
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);
	cinfo.dest = (struct jpeg_destination_mgr*)&jdest;
  
	cinfo.image_width = _w;
	cinfo.image_height = _h;
	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;
	jpeg_set_defaults(&cinfo);
	jpeg_set_quality(&cinfo, _q, TRUE);
	jpeg_start_compress(&cinfo, TRUE);
	row_pointer[0] = base;
	for(h = 0; h < _h; h++){
		jpeg_write_scanlines(&cinfo, row_pointer, 1);
		row_pointer[0] += pitch;
	}
	jpeg_finish_compress(&cinfo);
	jpeg_destroy_compress(&cinfo);
}

int SDLSaveJPG_GL_RW(Uint8* ptr, int w, int h, SDL_RWops* dst, int quality, int mustfree)
{
	// XXX assume pitch = w * 3

	saveJPG_core(ptr + (w * 3 * (h - 1)), -w * 3, dst, w, h, quality);
	if(mustfree){
		SDL_RWclose(dst);
	}
	return 0;
}

int SDLSaveJPG_RW(SDL_Surface* saveme, SDL_RWops* dst, int quality, int mustfree)
{
	SDL_Rect rect;
	SDL_Surface* src;
	src = SDL_CreateRGBSurface(SDL_SWSURFACE,
				   saveme->w, saveme->h, 24,
				   RMASK, 
				   GMASK, 
				   BMASK,
				   0);
	if(src == NULL) return -1;
	rect.x = rect.y = 0;
	rect.w = saveme->w;
	rect.h = saveme->h;
	if(SDL_LowerBlit(saveme, &rect, src, &rect) < 0){
		SDL_FreeSurface(src);
		SDL_SetError("Couldn't convert image to 24 bpp");
		return -1;
	}
	if(SDL_LockSurface(src) != 0) return -1;

	saveJPG_core(src->pixels, src->pitch, dst, src->w, src->h, quality);

	SDL_FreeSurface(src);
	if(mustfree) {
		SDL_RWclose(dst);
	}
	return 0;
}

#endif // USE_JPG
