/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
#ifndef __SDLSAVEMOVIE_H
#define __SDLSAVEMOVIE_H

#include <SDL.h>

#define SDLSAVEMOVIE_MODEL_THREAD 0
#define SDLSAVEMOVIE_MODEL_PUMP 1

typedef struct {
	Uint32 interval;  /* rw:  */
	int quality;  /* rw:  */
	int threadingmodel;  /* rw: SDLSAVEMOVIE_MODE_{THREAD|PUMP} */
	int useaudio; /* rw: 0 = no, 1 = yes */
	char* savepath;  /* :  */
	char* drivername;  /* rw:  */
	int driver_fallback;  /* rw: faillback if specified driver failed: 0 = no / 1 = yes */
	int rwops_needsfree;  /* rw: specified rwops needs to be freed when terminating: 0 = no / 1 = yes */
	struct SDLSaveMovie_Private* private;  /* driver specific */
	struct SDLSaveMovie_context* internal; /* internal usage */
} SDLSaveMovie;

struct SDLSaveMovie_Private;

extern int SDLSaveMovie_Start_SDL(SDLSaveMovie* param, SDL_Surface* src, char* filename);
extern int SDLSaveMovie_Start_SDL_RW(SDLSaveMovie* param, SDL_Surface* src, SDL_RWops* rwops);

extern int SDLSaveMovie_Start_GL(SDLSaveMovie* param, SDL_Rect* rect, Uint8 r, Uint8 g, Uint8 b, char* filename);

extern int SDLSaveMovie_Start_GL_RW(SDLSaveMovie* param, SDL_Rect* rect, Uint8 r, Uint8 g, Uint8 b, SDL_RWops* rwops);

extern void SDLSaveMovie_Update(SDLSaveMovie* param);
extern void SDLSaveMovie_Stop(SDLSaveMovie* param);

/* Parameter Settings */
extern SDLSaveMovie* SDLSaveMovie_Init();
extern void SDLSaveMovie_Terminate(SDLSaveMovie* param);

extern void SDLSaveMovie_MPEG_setMethod(SDLSaveMovie* ctx, char* method);
extern char* SDLSaveMovie_MPEG_getMethod(SDLSaveMovie* ctx);

/* Error Handling (global) */
extern char* SDLSaveMovie_GetError();
extern void SDLSaveMovie_SetError(char* message);

#endif
