/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
#ifndef __SDLSAVEMOVIE_INT_H
#define __SDLSAVEMOVIE_INT_H
#include <SDL.h>
#include "SDLSaveMovie.h"

#define SDLSAVEMOVIE_SRC_SDLSURFACE 0
#define SDLSAVEMOVIE_SRC_OPENGL 1

#define SDLSAVEMOVIE_REQUEST_RAW 0
#define SDLSAVEMOVIE_REQUEST_JPG 1

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
#define RMASK 0x000000ff
#define GMASK 0x0000ff00
#define BMASK 0x00ff0000
#else
#define RMASK 0x00ff0000
#define GMASK 0x0000ff00
#define BMASK 0x000000ff
#endif

typedef struct {
	char* identifier;
	int (*callback_init)(SDLSaveMovie* ctx);
	void (*callback_main)(SDLSaveMovie* ctx);
	void (*callback_terminate)(SDLSaveMovie* ctx);
	char* (*callback_getextension)();
	int (*callback_getprivatesize)();
} callback_vector;

struct SDLSaveMovie_context {
	int srctype;
	int requesttype;
	void* ptr;
	int size;
	SDL_Rect rect;
	int pitch;
	Uint8 r, g, b;
	SDL_RWops* rwops;

	/* thread context */
	int running;
	struct SDL_Thread* mythread;

	union {
		SDL_Surface* surface;
		Uint8* glpixels;
	} source; // should be distinguished by srctype

	Uint8* jpgtmp;
	SDL_RWops* rwmem;
	callback_vector* callback; // = NULL

};
typedef struct SDLSaveMovie_context SDLSaveMovie_context;

#ifdef USE_SWF
extern callback_vector SDLSaveMovie_SWF_vector;
#endif
#ifdef USE_MNG
extern callback_vector SDLSaveMovie_MNG_vector;
#endif
#ifdef USE_MPEG
extern callback_vector SDLSaveMovie_MPEG_vector;
#endif

#endif
