/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
#ifndef USE_JPG
#error "assertion failed: you can't use MNG driver without JPEG support."
#endif
#ifndef USE_MNG
#error "assertion failed: this source should'nt be compiled."
#endif
#include <SDL.h>

#include "../SDLSaveMovie.h"
#include "../SDLSaveMovie_int.h"
#include <stdlib.h>

#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#ifndef MNG_SUPPORT_WRITE
#define MNG_SUPPORT_WRITE
#endif
#ifndef MNG_ACCESS_CHUNKS
#define MNG_ACCESS_CHUNKS
#endif

#include <libmng.h>

typedef struct {
	SDLSaveMovie* param;
} userdata;

#ifndef O_BINARY
#define O_BINARY 0
#endif

/* callbacks for libmng */
mng_ptr myalloc (mng_size_t iSize){ return (mng_ptr)calloc (1, iSize); }
void myfree (mng_ptr pPtr, mng_size_t iSize){ free (pPtr); }
mng_bool myopenstream (mng_handle hMNG){
	return MNG_TRUE;
}
mng_bool myclosestream (mng_handle hMNG){
	return MNG_TRUE;
}
mng_bool mywritedata (mng_handle hMNG, mng_ptr pBuf,
                      mng_uint32 iSize, mng_uint32 *iWritten)
{
	userdata* p;
	p = (userdata*) mng_get_userdata(hMNG);
	*iWritten = SDL_RWwrite(p->param->internal->rwops, pBuf, 1, iSize);
	return MNG_TRUE;
}


static mng_handle hMNG;
static userdata data;


static int callback_init(SDLSaveMovie* param)
{
	SDLSaveMovie_context* ctx = param->internal;

	ctx->requesttype = SDLSAVEMOVIE_REQUEST_JPG;

	data.param = param;
	hMNG = mng_initialize((mng_ptr)&data, myalloc, myfree, MNG_NULL);
	if(!hMNG) {
		SDLSaveMovie_SetError("cannot initialize libmng");
		return -1;
	}
	if(mng_setcb_openstream(hMNG, myopenstream) != 0 ||
	   mng_setcb_closestream(hMNG, myclosestream) != 0 ||
	   mng_setcb_writedata(hMNG, mywritedata) != 0){
		SDLSaveMovie_SetError("cannot set callbacks.");
		return -1;
	}
	if(mng_create(hMNG) != 0){
		SDLSaveMovie_SetError("cannot create mng.");
		return -1;
	}
	if(mng_putchunk_mhdr(hMNG, ctx->rect.w, ctx->rect.h, 1000 / param->interval,
			     0, 0, 0, 0x157) != 0){
		SDLSaveMovie_SetError("putchunk_mhdr failed.");
		return -1;
	}
	return 0;
}

static void callback_main(SDLSaveMovie* param)
{
	SDLSaveMovie_context* ctx = param->internal;
	static Uint16 count = 1;
	mng_putchunk_defi(hMNG, count, 0, 0, MNG_FALSE, 0, 0, MNG_FALSE, 0, 0, 0, 0);
	count++;
	mng_putchunk_jhdr(hMNG, ctx->rect.w, ctx->rect.h, 10, 8, 8, 0, 0, 0, 0, 0);
	mng_putchunk_jdat(hMNG, ctx->size, ctx->ptr);
	mng_putchunk_iend(hMNG);
}

static void callback_terminate(SDLSaveMovie* param)
{
	/*  SDLSaveMovie_context* ctx = param->internal; */

	mng_putchunk_save(hMNG, MNG_TRUE, 0, 0);
	mng_putchunk_term(hMNG, 3, 0, 120, 0x7fffffff);
	mng_putchunk_seek(hMNG, 0, "");
	mng_putchunk_show(hMNG, MNG_TRUE, 0, 0, 0);
	mng_putchunk_mend(hMNG);

	mng_write(hMNG);
	mng_cleanup(&hMNG);
	fprintf(stderr, "cleanup.\n");
}

static char* callback_getextension()
{
	return "mng";
}

static int callback_getprivatesize()
{
	return 0;
}

callback_vector SDLSaveMovie_MNG_vector = {
	"mng",
	callback_init,
	callback_main,
	callback_terminate,
	callback_getextension,
	callback_getprivatesize,
};

