/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
#ifndef USE_MPEG
#error "assertion failed: this source should'nt be compiled."
#endif

#include <SDL.h>

#include "../SDLSaveMovie.h"
#include "../SDLSaveMovie_int.h"

#include <stdlib.h>

#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#include <fame.h>

#ifndef O_BINARY
#define O_BINARY 0
#endif


#define BUFFER_SIZE 0x10000

static fame_context_t* fame;
static Uint8* buf = NULL;
static fame_yuv_t yuv;
static SDL_Surface* tmp;

struct SDLSaveMovie_Private {
	char* method; /* mpeg1 or mpeg4 */
};

void SDLSaveMovie_MPEG_setMethod_private(SDLSaveMovie* param, char* method)
{
	param->private->method = method;
}

char* SDLSaveMovie_MPEG_getMethod_private(SDLSaveMovie* param)
{
	return param->private->method;
}

static int callback_init(SDLSaveMovie* param)
{
	SDLSaveMovie_context* ctx = param->internal;
	fame_parameters_t fparam = FAME_PARAMETERS_INITIALIZER;

	ctx->requesttype = SDLSAVEMOVIE_REQUEST_RAW;

	buf = (Uint8*)malloc(BUFFER_SIZE);
	if(buf == NULL) return -1;
	yuv.w = ctx->rect.w;
	yuv.h = ctx->rect.h;
	yuv.p = ctx->rect.w;

	yuv.y = (unsigned char*)malloc(ctx->pitch * ctx->rect.h);
	if(yuv.y == NULL) return -1;
	yuv.u = yuv.y + ctx->rect.w * ctx->rect.h;
	yuv.v = yuv.u + ctx->rect.w * ctx->rect.h / 4;

	if(ctx->srctype == SDLSAVEMOVIE_SRC_SDLSURFACE){
		tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,
					   ctx->rect.w, ctx->rect.h, 24,
					   RMASK,
					   GMASK,
					   BMASK,
					   0);
		if(tmp == NULL) return -1;
	}

#if 0
	fd = open(ctx->filename, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, 0644);
	if(fd < 0){
		SDLSaveMovie_SetError("can't open output file.");
		return -1;
	}
#endif
	fame = fame_open();
	if(fame == NULL){
		fprintf(stderr, "couldn't open fame.");
		return -1;
	}

	fparam.width = ctx->rect.w;
	fparam.height = ctx->rect.h;
	fparam.coding = "P";
	fparam.quality = param->quality;
	fparam.slices_per_frame = 1;
	fparam.frame_rate_num = 1000 / param->interval;
	fparam.frame_rate_den = 1;
	fparam.verbose = 0;
	fparam.profile = param->private->method;
	if(fparam.profile == NULL) fparam.profile = NULL;

	fame_init(fame, &fparam, buf, BUFFER_SIZE);

	return 0;
}

static void construct_fame_yuv(SDLSaveMovie_context* ctx, 
			       Uint8* ptr,
			       int direction,
			       int pitch,
			       fame_yuv_t* yuv)
{
	int x, y, realy;
	Uint8* p = ptr;

	Uint8* Y = (Uint8*)yuv->y;
	Uint8* Cr = (Uint8*)yuv->v;
	Uint8* Cb = (Uint8*)yuv->u;

	Uint32 R, G, B;
	int tY, tCr, tCb;

	for(y = 0; y < ctx->rect.h; y++){
		realy = y;
		if(direction == -1){ realy = ctx->rect.h - y - 1; }

		p = ptr + pitch * realy;

		/*
		  Y     0.2990    0.58700   0.11400            R
		  Cb = -0.1687   -0.33126   0.5           *    G
		  Cr    0.5      -0.41869  -0.08131           B

		*/
		for(x = 0; x < ctx->rect.w; x++){
			R = (Uint32)*(p++);
			G = (Uint32)*(p++);
			B = (Uint32)*(p++);

			tY = ((306 * R + 601 * G + 117 * B) + 512)/ 1024;
			if(tY > 255) *Y = 255;
			else if(tY < 0) *Y = 0;
			else *Y = (Uint8)tY;
			Y++;

			if(x & 1){
				if(realy & 1){
					tCb = ((-173 * R - 339 * G + 512 * B) + 131584)/ 1024;
					if(tCb > 255) *Cb = 255;
					else if(tCb < 0) *Cb = 0;
					else *Cb = (Uint8)tCb;
					Cb++;

					tCr = ((512 * R - 429 * G - 83 * B) + 131584)/ 1024;
					if(tCr > 255) *Cr = 255;
					else if(tCr < 0) *Cr = 0;
					else *Cr = (Uint8)tCr;
					Cr++;
				}
			}
		}
	}
  
}

static void callback_main(SDLSaveMovie* param)
{
	int i;
	Uint8* ptr; int direction; int pitch;
	static SDL_Rect rect;
	static SDL_Surface* saveme;

	SDLSaveMovie_context* ctx = param->internal;
	switch(ctx->srctype){
	case SDLSAVEMOVIE_SRC_SDLSURFACE:
		saveme = (SDL_Surface*)ctx->ptr;
		rect.x = rect.y = 0;
		rect.w = saveme->w; rect.h = saveme->h;
		if(SDL_LowerBlit(saveme, &rect, tmp, &rect) < 0){
			SDL_FreeSurface(tmp);
			return;
		}
		SDL_LockSurface(tmp);
		ptr = tmp->pixels;
		pitch = tmp->pitch;
		direction = 1;
		break;
	case SDLSAVEMOVIE_SRC_OPENGL:
#ifdef USE_OPENGL
		ptr = ctx->ptr;
		pitch = ctx->pitch;
		direction = -1;
		break;
#else
		fprintf(stderr, "SDLSaveMovie_MPEG: *fatal* you can't grab OpenGL pixmap.\n");
		return ;
		break; /* NOTREACHED */
#endif
	}

	construct_fame_yuv(ctx, ptr, direction, pitch, &yuv);
	fame_start_frame(fame, &yuv, NULL);
	while((i = fame_encode_slice(fame)) > 0){
		SDL_RWwrite(ctx->rwops, buf, 1, i);
		//    write(fd, buf, i);
	}
	fame_end_frame(fame, NULL);

	if(ctx->srctype == SDLSAVEMOVIE_SRC_SDLSURFACE){
		SDL_UnlockSurface(tmp);
	}
}

static void callback_terminate(SDLSaveMovie* param)
{
	int i;
	SDLSaveMovie_context* ctx = param->internal;

	i = fame_close(fame);
	if(i > 0){
		SDL_RWwrite(ctx->rwops, buf, 1, i);
		//    write(fd, buf, i);
	}
	//  close(fd);
	if(ctx->srctype == SDLSAVEMOVIE_SRC_SDLSURFACE){
		SDL_FreeSurface(tmp);
	}
	free(buf);
	free(yuv.y);
}

static char* callback_getextension()
{
	return "mpeg";
}

static int callback_getprivatesize()
{
	return sizeof(struct SDLSaveMovie_Private);
}
    
callback_vector SDLSaveMovie_MPEG_vector = {
	"mpeg",
	callback_init,
	callback_main,
	callback_terminate,
	callback_getextension,
	callback_getprivatesize,
};
