/*
 * Copyright (c) 2002 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
#ifndef USE_JPG
#error "assertion failed: you can't use SWF driver without JPEG support."
#endif
#ifndef USE_SWF
#error "assertion failed: this source should'nt be compiled."
#endif

#include <SDL.h>
#include <stdlib.h>
#include <ming.h>
#include "../SDLSaveMovie.h"
#include "../SDLSaveMovie_int.h"

static SWFMovie movie;

static void output_RW(byte b, void* data)
{
	SDL_RWops* rwops = (SDL_RWops*)data;
	SDL_RWwrite(rwops, &b, 1, 1);
}

static int callback_init(SDLSaveMovie* param)
{
	SDLSaveMovie_context* ctx = param->internal;
	ctx->requesttype = SDLSAVEMOVIE_REQUEST_JPG;

	/* prepare movie */
	movie = newSWFMovie();
	if(movie == NULL) return -1;
	SWFMovie_setRate(movie, 1000.0 / (float)(param->interval));
	SWFMovie_setDimension(movie, ctx->rect.w, ctx->rect.h);
	SWFMovie_setBackground(movie, ctx->r, ctx->g, ctx->b);
	return 0;
}

static void callback_main(SDLSaveMovie* param)
{
	SWFBitmap swfbitmap;
	SWFInput swfinput;
	Uint8* bmp;
	SDLSaveMovie_context* ctx = param->internal;

	bmp = (Uint8*)malloc(ctx->size);
	memcpy(bmp, ctx->ptr, ctx->size);

	swfinput = newSWFInput_allocedBuffer(bmp, ctx->size);
	if(swfinput == NULL){
		fprintf(stderr, "newSWFInput_buffer:\n");
		return;
	}
	swfbitmap = newSWFBitmap_fromInput(swfinput);
	if(swfbitmap == NULL){
		fprintf(stderr, "newSWFBitmap_fromInput: \n");
		return;
	}
	if(SWFMovie_add(movie, swfbitmap) == NULL){
		fprintf(stderr, "SWFMovie_add failed.\n");
		return ;
	}
	SWFMovie_nextFrame(movie);
}

static void callback_terminate(SDLSaveMovie* param)
{
	SWFMovie_output(movie, output_RW, param->internal->rwops);
}

static char* callback_getextension()
{
	return "swf";
}

static int callback_getprivatesize()
{
	return 0;
}

callback_vector SDLSaveMovie_SWF_vector = {
	"swf",
	callback_init,
	callback_main,
	callback_terminate,
	callback_getextension,
	callback_getprivatesize,
};

